#include <stdbool.h>
#include <xc.h>
#include "adc.h"
#include "midi.h"
#include "inputs.h"

#define _XTAL_FREQ 20000000

// Store for rolling average values
char analogMemory[A_INPUTS][8];

void initADC()
{
    // AN0 as analog input
    // Voltage Reference set to Vss, Vdd
    ADCON1 = 0b00001110;
    
    // Acquisition time - 20Tad
    // ADC clock - Fosc/64 - 20Mhz
    // AD Result format - Right justified
    ADCON2 = 0b10111110;
    
    // Turn on ADC module
    ADCON0bits.ADON = 1;
    __delay_us(1);
}

char readADC(char chanel)
{
    int value = 0;
    
    setChanel(chanel);

    // start conversion
    ADCON0bits.GODONE = 1;
    
    while (ADCON0bits.GODONE);
    
    // Convert to 8 bit
    value = ((ADRESH << 6) | (ADRESL >> 2));

    // Shift to 7 bit
    value = value >> 1;
    
    return value;
}

void setChanel(char chanel)
{ 
    ADCON0bits.CHS = chanel;
}


// Rolling average 
char getADC(char chanel, char adr)
{
    char value, i;
    long sum;
    
    setAdress(adr);
    
    value = readADC(chanel);
                
    // Shift last 4 values in array in down direction and make sum 
    sum = 0;
    for (i = 7; i > 0; --i)
    {
        analogMemory[adr][i] = analogMemory[adr][i-1];
        sum += (unsigned long) analogMemory[adr][i];
    }

    // put last measured value to the begin of the array 
    analogMemory[adr][0] = value;

    // make sum for all 8 values
    sum += (unsigned long) value;

    // calculate average of last 8 measured values
    value = (unsigned char) (sum >> 3);
    
    return value;
}